/*
   tsc2046 TouchPannel on ILI9341 LCD for STM32F4xx-nucleo

     Copyright (C) by Y.Ishioka
*/
#include "stm32f4xx.h"
#include "spi.h"

#define START  0x80 
#define Z2POS  0x40 
#define Z1POS  0x30 
#define XPOS   0x50 
#define YPOS   0x10 
#define PDON   0x03 

void  delay( int  msec );

/*
 * return  0: no touch
 *         1: touch
 */
int  lcdtp_read_irq( void )
{
  int  stat = 0;
  unsigned int  data;

  data = GPIOA->IDR ;
  
  if( (data & 0x0400) == 0 ) {
    stat = 1;
  }

  return  stat;
}


int  lcdtp_read_posi( int *x, int *y )
{
  int  x0, x1;
  int  y0, y1;
  int  stat;

  stat = lcdtp_read_irq();
  //stat = 1;
  if( stat != 0 ) {
    xspi_set_cs_tp(0); /* CS:0 */
    xspi_send( START|YPOS );
    y0 = (int)xspi_recv();
    y1 = (int)xspi_recv();
    xspi_set_cs_tp(1); /* CS:1 */

    xspi_set_cs_tp(0); /* CS:0 */
    xspi_send( START|XPOS );
    x0 = (int)xspi_recv();
    x1 = (int)xspi_recv();
    xspi_set_cs_tp(1); /* CS:1 */

    *x = ((x0 << 4) | (x1 >> 4)) & 0xfff;
    *y = ((y0 << 4) | (y1 >> 4)) & 0xfff;

    /* powerdown */
    xspi_set_cs_tp(0); /* CS:0 */
    xspi_send( START|XPOS );
    xspi_set_cs_tp(1); /* CS:1 */

  }

  return  stat;
}


void  lcdtp_dev_init( void )
{
    /* set clock enable */
    RCC->AHB1ENR |= (  RCC_AHB1ENR_GPIOAEN
                    ) ;

    /** for LCD_TP_TIRQ:PA10 **/
    GPIOA->MODER   &= ~(0x00300000);
    GPIOA->OTYPER  &= ~(0x0400);
    GPIOA->OSPEEDR &= ~(0x00300000);
    GPIOA->OSPEEDR |=  (0x00200000);
    GPIOA->PUPDR   &= ~(0x00300000);
    //GPIOA->PUPDR   |=  (0x00100000); /* pull-up */
}


void  lcdtp_init( void )
{
  lcdtp_dev_init();

  /* powerdown */
  xspi_set_cs_tp(0); /* CS:0 */
  xspi_send( START|XPOS );
  xspi_set_cs_tp(1); /* CS:1 */
}
