/*
   MojiAsobi for STM32F4xx-nucleo

     Copyright (C) by Y.Ishioka
*/

#include <stm32f4xx.h>
#include "kernel.h"
#include "syssvc/syslog.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ff.h"
#include "diskio.h"

#include "matled.h"

void  xspi_init( void );
void  lcdtp_init( void );
int  lcdtp_read_posi( int *x, int *y );
void  nfc_init( void );
int  nfc_get_uid( unsigned char *uid, unsigned char *uidLength );
void  lcd_init( void );
void  lcd_setblock( unsigned short int x0, unsigned short int x1, unsigned short int y0, unsigned short int y1 );
void  lcd_pset( int x, int y, int color );
void  lcd_pset_fonta( unsigned char *font, int x, int y, int fg, int bg );
void  lcd_pset_fontk( unsigned char *font, int x, int y, int fg, int bg );
void  lcd_put_string( int x, int y, int fg, int bg, unsigned char *msg, int size );
void  lcd_fill_full( int color );
unsigned char  *read_fontx2_a( unsigned int code );
unsigned char  *read_fontx2_k( unsigned int code );


#define DEF_MAX_CHARLEN   32
#define DEF_BUFF_ALL_COL  ((2+DEF_MAX_CHARLEN+2)*32)
#define DEF_ID_STR_SIZE   (3 * 8)
#define DEF_MSG_BUF_SIZE  36
#define DEF_MSG_BAG_NUM   20
#define DEF_FONT_SIZE     16
#define DEF_BUF_SIZE      (1024 * 4)

#define  DEF_ITEM_MERGIN_X  20
#define  DEF_ITEM_MERGIN_Y  30

#define DEF_FILE_MASTER   "master.txt"
#define DEF_FILE_MASTER2  "master.txt"
#define DEF_FILE_MSGTXT   "msg.txt"


int  name_main_flag;
int  name_out_col_max;
int  name_out_color;
int  name_msg_renew;
int  name_msg_stop;
int  demo_item_mark;
unsigned char  demo_uidLength;

struct msg_bag_tag {
  char           id[ DEF_ID_STR_SIZE ];
  unsigned char  msg[ DEF_MSG_BUF_SIZE ];
};

struct  tp_posi_tag {
  int  x;
  int  y;
};

struct msg_bag_tag  name_msg_bag[ DEF_MSG_BAG_NUM ];
unsigned char  name_msg_list[ DEF_MSG_BAG_NUM ][ DEF_MAX_CHARLEN+1 ];
char  demo_buff[ 128 ];
char  demo_id_buff[ DEF_ID_STR_SIZE ];
unsigned char  matled_out_buff[32][DEF_BUFF_ALL_COL];
unsigned char  tmp_buff[ DEF_BUF_SIZE ];


static  unsigned char  dat_bit[8] = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 } ;
uint16_t  name_out_color_dat[7] = { 0x01, 0x02, 0x04, 0x03, 0x05, 0x06, 0x07 };

                                                 /* 123456789012345 */
unsigned char  *name_def_regist = (unsigned char *)"y Ƃ낭 z";
unsigned char  *name_def_delete = (unsigned char *)"y  z";
unsigned char  *def00_msg = (unsigned char *)"   ";

struct  tp_posi_tag  tp_posi_regist0 = {  339, 1736 };
struct  tp_posi_tag  tp_posi_regist1 = {  912, 1604 };
struct  tp_posi_tag  tp_posi_delete0 = { 1106, 1623 };
struct  tp_posi_tag  tp_posi_delete1 = { 1632, 1440 };

struct  tp_posi_tag  tp_posi_item[ DEF_MSG_BAG_NUM ] = {
  {  352, 1370 },
  {  370, 1296 },
  {  368, 1242 },
  {  376, 1176 },
  {  363, 1077 },
  {  349, 1000 },
  {  360,  923 },
  {  373,  832 },
  {  352,  744 },
  {  364,  657 },

  { 1072, 1240 },
  { 1060, 1155 },
  { 1066, 1072 },
  { 1058,  992 },
  { 1056,  912 },
  { 1056,  824 },
  { 1042,  736 },
  { 1040,  651 },
  { 1013,  579 },
  { 1011,  499 },
};


/********** for spirntf() **********/
unsigned int  sbrk_heap[ 0x1000 ];

void  *_sbrk( int incr )
{
  return  &sbrk_heap[ 0x1000 ];
}
/********** for spirntf() **********/


void  delay( int msec )
{
  tslp_tsk( msec );
}


void  name_syslog_posi( int x0, int y0, int x1, int y1 )
{
  syslog( LOG_NOTICE, "( %d, %d )-( %d, %d )", x0, y0, x1, y1 );
}


/****************************************************/
void  demo_disp_stat_clear( void )
{
                   /* 12345678901234567890123456789012 */
  const char *stat = "STAT:                           ";

  lcd_put_string(   0, 60, 0xffe0, 0x2104, (unsigned char *)stat, strlen(stat) );
}


/****************************************************/
void  demo_disp_stat( char *msg )
{
  char  buff[ 64 ];
  const char *stat = "STAT:";

  demo_disp_stat_clear();

  sprintf( buff, "%s %s", stat, msg );
  lcd_put_string(   0, 60, 0xffe0, 0x2104, (unsigned char *)buff, strlen(buff) );
}


/****************************************************/
void  demo_master_msgbag( int len )
{
  unsigned char  *src, *dst;
  unsigned char  data;
  int  cnt, all;
  int  i;
  char  buff[ 64 ];

  all = 0;
  src = tmp_buff;
  for( i=0 ; i<DEF_MSG_BAG_NUM ; i++ ) {
    if( all >= len ) {
      break;
    }
    if( *src < 0x30 || 0x7f < *src ) {
      break;
    }

    dst = (unsigned char *)name_msg_bag[i].id;
    cnt = 0;
    while( (data = *src++) != 0x0a ) {
      if( data == 0x0d ) {
        continue;
      }
      all++;
      cnt++;
      if( cnt < DEF_ID_STR_SIZE - 1 ) {
        if( 'A' <= data && data <= 'Z' ) {
          data += 'a' - 'A';
        }
        *dst++ = data;
      }
    }
    *dst = 0x00;  /* set nul */

    dst = name_msg_bag[i].msg;
    cnt = 0;
    while( (data = *src++) != 0x0a ) {
      if( data == 0x0d ) {
        continue;
      }
      all++;
      cnt++;
      if( cnt < DEF_MAX_CHARLEN ) {
        *dst++ = data;
      }
    }
    *dst = 0x00;  /* set nul */

    //syslog( 5, "(%2d) id[0]=0x%02x  msg[0]=0x%02x", i, name_msg_bag[i].id[0], name_msg_bag[i].msg[0] );
  }
  sprintf( buff, "Read %d item.", i );
  demo_disp_stat( buff );
}


/****************************************************/
void  demo_read_master( void )
{
  FRESULT  ret;
  FATFS  fs;
  FIL  fil;
  UINT  dsz;

  syslog( LOG_NOTICE, "f_mount()" );
  ret = f_mount( 0, &fs );
  if( ret != FR_OK ) {
    syslog( LOG_NOTICE, "ERROR f_mount() = %d", ret );
  }

  syslog( LOG_NOTICE, "f_open()" );
  ret = f_open( &fil, DEF_FILE_MASTER, FA_READ );
  if( ret != FR_OK ) {
    syslog( LOG_NOTICE, "ERROR f_open(%s) = %d", tmp_buff, ret );
  }

  ret = f_read( &fil, tmp_buff, DEF_BUF_SIZE, &dsz );
  syslog( LOG_NOTICE, "f_read()  dsz=%d", dsz );
  if( ret != FR_OK ) {
    syslog( LOG_NOTICE, "ERROR f_read(%s) = %d", tmp_buff, ret );
  }

  demo_master_msgbag( dsz );

  syslog( LOG_NOTICE, "f_close()" );
  f_close( &fil );
}


/****************************************************/
char  name_data_crlf[ 2 ] = { 0x0d, 0x0a };

void  demo_write_master( void )
{
  FRESULT  ret;
  FATFS  fs;
  FIL  fil;
  UINT  dsz, all;
  int  i;
  int  cnt;
  char  buff[ 64 ];

  name_msg_stop = 1;
  tslp_tsk( 10 );

  syslog( LOG_NOTICE, "f_mount()" );
  ret = f_mount( 0, &fs );
  if( ret != FR_OK ) {
    syslog( LOG_NOTICE, "ERROR f_mount() = %d", ret );
  }

  syslog( LOG_NOTICE, "f_open()" );
  ret = f_open( &fil, DEF_FILE_MASTER2, FA_CREATE_ALWAYS | FA_WRITE );
  if( ret != FR_OK ) {
    syslog( LOG_NOTICE, "ERROR f_open(%s) = %d", DEF_FILE_MASTER2, ret );
  }

  all = 0;
  cnt = 0;
  for( i=0 ; i<DEF_MSG_BAG_NUM ; i++ ) {
    if( name_msg_bag[i].id[0] == 0 ) {
      continue;
    }
    syslog( LOG_NOTICE, "f_write(id,msg,%d)", i );
    ret = f_write( &fil, name_msg_bag[i].id, strlen(name_msg_bag[i].id), &dsz );
    all += dsz;
    if( ret != FR_OK ) {
      syslog( LOG_NOTICE, "ERROR f_write(%s) = %d", DEF_FILE_MASTER2, ret );
    }
    ret = f_write( &fil, name_data_crlf, 2, &dsz );
    all += dsz;
    if( ret != FR_OK ) {
      syslog( LOG_NOTICE, "ERROR f_write(crlf) = %d", ret );
    }
    ret = f_write( &fil, name_msg_bag[i].msg, strlen((char *)name_msg_bag[i].msg), &dsz );
    all += dsz;
    if( ret != FR_OK ) {
      syslog( LOG_NOTICE, "ERROR f_write(%s) = %d", DEF_FILE_MASTER2, ret );
    }
    ret = f_write( &fil, name_data_crlf, 2, &dsz );
    all += dsz;
    if( ret != FR_OK ) {
      syslog( LOG_NOTICE, "ERROR f_write(crlf) = %d", ret );
    }
    cnt++;
  }
  syslog( LOG_NOTICE, "demo_write_master()  all=%d", all );

  syslog( LOG_NOTICE, "f_close()" );
  f_close( &fil );

  sprintf( buff, "Save %d item.", cnt );
  demo_disp_stat( buff );

  name_msg_stop = 0;
}


/****************************************************/
void  demo_msgtxt_msglst( int len )
{
  unsigned char  *src, *dst;
  unsigned char  data;
  int  cnt, all;
  int  i;

  all = 0;
  src = tmp_buff;
  for( i=0 ; i<DEF_MSG_BAG_NUM ; i++ ) {
    if( all >= len ) {
      break;
    }
    dst = name_msg_list[i];
    cnt = 0;
    while( (data = *src++) != 0x0a ) {
      cnt++;
      all++;
      if( data == 0x0d ) {
        continue;
      }
      if( cnt < DEF_MAX_CHARLEN ) {
        *dst++ = data;
      }
    }
    *dst = 0x00;  /* set nul */
  }
}


/****************************************************/
void  demo_read_msgtxt( void )
{
    FRESULT  ret;
    FATFS  fs;
    FIL  fil;
    UINT  dsz;

    syslog( LOG_NOTICE, "f_mount()" );
    ret = f_mount( 0, &fs );
    if( ret != FR_OK ) {
        syslog( LOG_NOTICE, "ERROR f_mount() = %d", ret );
    }

    syslog( LOG_NOTICE, "f_open()" );
    ret = f_open( &fil, DEF_FILE_MSGTXT, FA_READ );
    if( ret != FR_OK ) {
        syslog( LOG_NOTICE, "ERROR f_open(%s) = %d", tmp_buff, ret );
    }

    ret = f_read( &fil, tmp_buff, DEF_BUF_SIZE, &dsz );
    syslog( LOG_NOTICE, "f_read()  dsz=%d", dsz );
    if( ret != FR_OK ) {
        syslog( LOG_NOTICE, "ERROR f_read(%s) = %d", tmp_buff, ret );
    }

    demo_msgtxt_msglst( dsz );

    syslog( LOG_NOTICE, "f_close()" );
    f_close( &fil );
}


/****************************************************/
void  name_matled_buff_clear( void )
{
  int  i, j;

  for( i=0 ; i<32 ; i++ ) {
    for( j=0 ; j<DEF_BUFF_ALL_COL ; j++ ) {
      matled_out_buff[i][j] = 0x00;
    }
  }
}


/****************************************************/
void  main_matled_pset_font( unsigned char *font, int x, int y, int width_b )
{
  int  i, j, k;

  for( i=0 ; i<DEF_FONT_SIZE ; i++ ) {  /* line */
    for( j=0 ; j<width_b ; j++ ) {      /* col */
      for( k=0 ; k<8 ; k++ ) {          /* bit pos */
        if( (font[(i*width_b)+j] & dat_bit[k]) != 0 ) {
          matled_out_buff[ y+i ][ DEF_DISP_WIDTH + x+j*8+k ] = 1;
        }
      }
    }
  }
}


/****************************************************/
void  main_demo_matled_out( char *id )
{
  int  pos ;
  int  size;
  unsigned char  *msg ;
  unsigned char  *fontdata ;
  unsigned int  code ;
  int  x, y;
  int  i;
  char  buff[ 64 ];

  msg = def00_msg;

  if( id != NULL ) {
    for( i=0 ; i<DEF_MSG_BAG_NUM ; i++ ) {
      if( name_msg_bag[i].id == 0 ) {
        continue;
      }
      if( strcmp( id, name_msg_bag[i].id ) == 0 ) {
        msg = name_msg_bag[i].msg;
        //syslog( 5, "id matched." );
        sprintf( buff, "id matched. No.%d", i );
        demo_disp_stat( buff );
        break;
      }
    }
  }

  x = 0;
  y = 8;

  size = strlen( (char *)msg );
  pos = 0 ;
  while( (code = msg[pos++]) != 0 ) {
    if( pos > size ) {
      break;
    }
    if( code < 0x80 ) {
      if( code == 0x0d || code == 0x0a ) {
        code = ' ' ;
      }
      fontdata = read_fontx2_a( code );
      main_matled_pset_font( fontdata, x, y, 1 );
      x += DEF_FONT_SIZE/2;
    } else {
      code = (code<<8) + msg[pos++] ;
      if( pos > size ) {
        break;
      }
      fontdata = read_fontx2_k( code );
      main_matled_pset_font( fontdata, x, y, 2 );
      x += DEF_FONT_SIZE;
    }
  }
  name_out_col_max = DEF_DISP_WIDTH + x;

  syslog( 5, "name_out_col_max=%d", name_out_col_max );
}


/****************************************************/
void  name_make_msgtxt_out( unsigned char *dst, unsigned char *src, int max )
{
  int  i;
  unsigned char  data;

  for( i=0 ; i<max ; i++ ) {
    data = *src++;
    if( data < 0x80 ) {
      /* for ASCII */
      *dst++ = data;
    } else {
      /* for 2byte code */
      if( i >= (max-1) ) {
        *dst = 0x00;   /* set nul code */
        break;
      }
      *dst++ = data;
      *dst++ = *src++;
      i++;
    }
  }
}

/****************************************************/
void  name_msgtxt_out( void )
{
  int  y;
  int  i;
  unsigned char  temp[ 16 ];

  y = 120;
  for( i=0 ; i<10 ; i++ ) {
    memset( temp, 0x00, 16 );
    name_make_msgtxt_out( temp, name_msg_list[i], 11 );
    sprintf( demo_buff, " %s", temp );
    lcd_put_string(   0, y, 0x07e0,  0x0000, (unsigned char *)demo_buff, strlen(demo_buff) );
    y += 20;
  }

  y = 120;
  for(  ; i<20 ; i++ ) {
    memset( temp, 0x00, 16 );
    name_make_msgtxt_out( temp, name_msg_list[i], 11 );
    sprintf( demo_buff, " %s", temp );
    lcd_put_string( 120, y, 0x07e0,  0x0000, (unsigned char *)demo_buff, strlen(demo_buff) );
    y += 20;
  }
}


/****************************************************/
void  sd_init( void )
{
  DSTATUS  rc;

  syslog( LOG_NOTICE, "disk_initialize()" );
  rc = disk_initialize( 0 );
  if( rc != RES_OK ) {
    syslog( LOG_NOTICE, "ERROR disk_initialize() = %d", rc );
  }
}


/****************************************************/
void  main_matled_init( void )
{
  name_out_col_max = 0;

  name_matled_buff_clear();
  main_demo_matled_out( NULL );
}


/****************************************************/
void  name_disp_regist( int fg, int bg )
{
  lcd_put_string(   0, 30, fg, bg, (unsigned char *)name_def_regist, strlen((char *)name_def_regist) );
}


void  name_disp_delete( int fg, int bg )
{
  lcd_put_string( 128, 30, fg, bg, (unsigned char *)name_def_delete, strlen((char *)name_def_delete) );
}


void  name_nfc_init( void )
{
  char  buff[ 64 ];

  sprintf( buff, ": -- -- -- -- -- -- -- --" );
  lcd_put_string( 16, 0, 0x07ff, 0x0000, (unsigned char *)buff, strlen(buff) );
  lcd_put_string(  0, 0, 0xffff, 0x0000, (unsigned char *)"ID", 2 );

  name_disp_regist( 0xffff, 0x0400 );
  name_disp_delete( 0xffff, 0x0010 );
}


/*
 *  item : 1 - 20
 */
void  name_disp_item_box( int item, unsigned char *mark, int size, int color )
{
  int  x, y;

  if( item <= 0 || 20 < item) {
    /* out of range */
    return ;
  }

  --item;

  if( item < 10 ) {
    x = 0;
    y = 120 + item * 20;
  } else if( item < 20) {
    item -= 10;
    x = 120;
    y = 120 + item * 20;
  }
    /* 0x07e0 */
  lcd_put_string( x, y, color,  0x0000, mark, size);
}


/*
 *  item : 1 - 20
 */
int  demo_item_regist( int item, char *id, int len )
{
  int  i;
  int  ret = 0;

  for( i=0 ; i<DEF_MSG_BAG_NUM ; i++ ) {
    if( strcmp( name_msg_bag[i].id, id ) == 0 ) {
      strcpy( (char *)name_msg_bag[i].msg, (char *)name_msg_list[item-1] );
      break;
    }
  }

  if( i == DEF_MSG_BAG_NUM ) {
    /* unmatched */
    for( i=0 ; i<DEF_MSG_BAG_NUM ; i++ ) {
      if( name_msg_bag[i].id[0] == 0 ) {
        strcpy( name_msg_bag[i].id, id );
        strcpy( (char *)name_msg_bag[i].msg, (char *)name_msg_list[item-1] );
        break;
      }
    }
    if( i == DEF_MSG_BAG_NUM ) {
      syslog( 5, "Fail. item full." );
      demo_disp_stat( "Fail. item full." );
      ret = 1;
    }
  }

  return  ret;
}


/*
 *  item : 1 - 20
 */
int  demo_item_delete( int item, char *id, int len )
{
  int  i;
  int  ret = 0;

  for( i=0 ; i<DEF_MSG_BAG_NUM ; i++ ) {
    if( strcmp( name_msg_bag[i].id, id ) == 0 ) {
      name_msg_bag[i].id[0] = 0;
      name_msg_bag[i].msg[0] = 0;
      break;
    }
  }

  if( i == DEF_MSG_BAG_NUM ) {
    /* unmatched */
    syslog( 5, "Fail. item unmatched." );
    demo_disp_stat( "Fail. item unmatched." );
    ret = 1;
  }

  return  ret;
}


int  name_tp_check( void )
{
  int  stat;
  int  x, y;
  int  i, j;
  int  ret = 0;
  int  res;
  static int  flag = 0;
  static int  before = 0;
  static int  repeat;
  char  buff[ 64 ];

  stat = lcdtp_read_posi( &x, &y );
  if( stat != 0 ) {
    sprintf( buff, "(%4d,%4d)", x, y );
    lcd_put_string( 150, 90, 0x2104, 0x0000, (unsigned char *)buff, strlen(buff) );

    /** check regist **/
    if( tp_posi_regist0.x <= x && x <= tp_posi_regist1.x
     && tp_posi_regist1.y <= y && y <= tp_posi_regist0.y ) {
      if( before != 30 ) {
        flag = 30;
        repeat = 0;
        name_disp_regist( 0xffe0, 0x4208 );
        if( demo_uidLength != 0 && demo_item_mark != 0 ) {
          res =demo_item_regist( demo_item_mark, demo_id_buff, demo_uidLength );
          if( res == 0 ) {
            demo_write_master();
            demo_disp_stat( "Regist complete." );
          }
          ret = 1;
        }
      } else {
        repeat++ ;
      }
    }

    /** check delete **/
    if( tp_posi_delete0.x <= x && x <= tp_posi_delete1.x
     && tp_posi_delete1.y <= y && y <= tp_posi_delete0.y ) {
      if( before != 31 ) {
        flag = 31;
        repeat = 0;
        name_disp_delete( 0xffe0, 0x4208 );
        if( demo_uidLength != 0 ) {
          res = demo_item_delete( demo_item_mark, demo_id_buff, demo_uidLength );
          if( res == 0 ) {
            demo_write_master();
            demo_disp_stat( "Delete complete." );
          }
          ret = 2;
        }
      } else {
        repeat++ ;
      }
    }

    /** check item **/
    for( i=0,j=1 ; i<DEF_MSG_BAG_NUM ; i++,j++ ) {
      if( tp_posi_item[i].x-DEF_ITEM_MERGIN_X <= x && x <= tp_posi_item[i].x+DEF_ITEM_MERGIN_X
       && tp_posi_item[i].y-DEF_ITEM_MERGIN_Y <= y && y <= tp_posi_item[i].y+DEF_ITEM_MERGIN_Y ) {
        if( before != j ) {
          flag = j;
          repeat = 0;
          if( demo_item_mark != 0 ) {
            name_disp_item_box( demo_item_mark, (unsigned char *)"", 2, 0x07e0 );
          }
          demo_item_mark = j;
          name_disp_item_box( demo_item_mark, (unsigned char *)"", 2, 0xffff );
        } else {
          repeat++ ;
        }
        break;
      }
    }

  } else {
    flag = 0;
  }

  if( before == 30 && flag != 30 ) {
    name_disp_regist( 0xffff, 0x0400 );
  }
  if( before == 31 && flag != 31 ) {
    name_disp_delete( 0xffff, 0x0010 );
  }

  before = flag;

  return  ret;
}


/****************************************************/
void  name_main( void )
{
  xspi_init();
  sd_init();
  lcd_init();
  lcdtp_init();
  nfc_init();
  matled_init();
  main_matled_init();
  name_nfc_init();

  name_main_flag = 0;
  while( 1 ) {
    tslp_tsk( 1000 );
  }
}


/****************************************************/
void  name_task1( void )
{
  int  col;
  int  step, temp;

  name_out_color = 0;
  name_msg_stop = 0;

  while( name_main_flag ) {
    tslp_tsk( 10 );
  }

  name_msg_renew = 0;

  col = 0;
  step = 0;
  while( 1 ) {
    tslp_tsk( 3 );

    if( name_msg_stop ) {
      continue;
    }

    if( name_msg_renew != 0 ) {
      col = 0;
      step = 0;
      name_msg_renew = 0;
      name_out_color++;
      if( name_out_color >= 7 ) {
        name_out_color = 0;
      }
    }

    if( btn_read() == 0 ) {
      temp = adc_read();
      if( temp > 200 ) {
          step += temp;
      }
      if( step > 5000 ) {
        step = 0;
        col++;
        if( col >= name_out_col_max ) {
          col = 0;
          name_out_color++;
          if( name_out_color >= 7 ) {
            name_out_color = 0;
          }
        }
      }
    }

    matled_dev_output_c( (unsigned char*)matled_out_buff, col, DEF_BUFF_ALL_COL,
                         name_out_color_dat[name_out_color] );
  }
}


/****************************************************/
void  name_task2( void )
{
  while( name_main_flag ) {
    tslp_tsk( 10 );
  }

  while( 1 ) {
    tslp_tsk( 1000 );
  }
}


void  name_task3( void )
{
  int  i;
  int  ret;
  int  flag;
  unsigned char uid[8];    // Buffer to store the returned UID

  while( name_main_flag ) {
    tslp_tsk( 10 );
  }

  for( i=0 ; i<DEF_MSG_BAG_NUM ; i++ ) {
    name_msg_bag[i].id[0]  = 0x00;
    name_msg_bag[i].msg[0] = 0x00;
  }
  demo_uidLength = 0;
  demo_item_mark = 0;

  demo_disp_stat_clear();

  demo_read_msgtxt();
  demo_read_master();
  name_msgtxt_out();

  flag = 0;
  while( 1 ) {
    ret = name_tp_check();
    if( ret != 0 ) {
      flag = 0;
    }

    ret = nfc_get_uid( uid, &demo_uidLength );
    if( ret != 0 ) {
     if( flag == 0 ) {
      switch( demo_uidLength ) {
      case 4:
        sprintf( demo_id_buff, "%02x %02x %02x %02x",
                 uid[0], uid[1], uid[2], uid[3] );
        sprintf( demo_buff, ": %s -- -- -- --", demo_id_buff );
        break;
      case 7:
        sprintf( demo_id_buff, "%02x %02x %02x %02x %02x %02x %02x",
                 uid[0], uid[1], uid[2], uid[3], uid[4], uid[5], uid[6] );
        sprintf( demo_buff, ": %s --", demo_id_buff );
        break;
      case 8:
        sprintf( demo_id_buff, "%02x %02x %02x %02x %02x %02x %02x %02x",
                 uid[0], uid[1], uid[2], uid[3], uid[4], uid[5], uid[6], uid[7] );
        sprintf( demo_buff, ": %s", demo_id_buff );
        break;
      default:
        sprintf( demo_id_buff, "-- -- -- -- -- -- -- --" );
        sprintf( demo_buff, ": %s", demo_id_buff );
        demo_uidLength = 0;
        break;
      }

      lcd_put_string( 16, 0, 0x07ff, 0x0000, (unsigned char *)demo_buff, strlen(demo_buff) );
      lcd_put_string(  0, 0, 0xf800, 0x0000, (unsigned char *)"ID", 2 );
      flag = 1;

      name_matled_buff_clear();
      main_demo_matled_out( demo_id_buff );
      name_msg_renew = 1;
     }
    } else {
      if( flag ) {
        lcd_put_string( 0, 0, 0xffff, 0x0000, (unsigned char *)"ID", 2 );
        flag = 0;

        name_matled_buff_clear();
        main_demo_matled_out( NULL );
        name_msg_renew = 1;
      }
    }

    tslp_tsk( 200 );
  }
}

